using System;
using System.Globalization;
using Microsoft.Practices.RepositoryFactory.CodeGeneration.ObjectModel;

namespace Microsoft.Practices.RepositoryFactory.CommonHelpers
{
    public static class NameResolver
    {
        /// <summary>
        /// Given a business entity, determines the folder name for the repository 
        /// and related classes that will be generated by the guidance package.
        /// </summary>
        /// <remarks>
        /// The following algorithm is applied:
        /// <para></para>
        /// Target folder name = entityName
        /// </remarks>
        public static string ResolveFolderName(string entityName)
        {
            if (String.IsNullOrEmpty(entityName)) throw new ArgumentNullException("entityName");

            return string.Format(CultureInfo.InvariantCulture, "{0}DataAccess", entityName);
        }

        /// <summary>
        /// Used by BusinessEntity.cs.t4 and BusinessEntity.vb.t4
        /// </summary>
        public static string GetFolderName(string rootNamespace, string entityNamespace)
        {

            string destinationNamespace = entityNamespace;
            string relativePath = string.Empty;
            string relativeNamespace = string.Empty;

            if (destinationNamespace != null)
            {
                int businessNamespaceLen = rootNamespace.Length;
                if (destinationNamespace.Length >= businessNamespaceLen)
                {
                    string leadDestNamespace = destinationNamespace.Substring(0, businessNamespaceLen);
                    if (leadDestNamespace.ToUpper() == rootNamespace.ToUpper())
                    {
                        if (destinationNamespace.Length == businessNamespaceLen)
                        {
                            relativeNamespace = string.Empty;
                        }
                        else
                        {
                            relativeNamespace = destinationNamespace.Substring(businessNamespaceLen + 1, destinationNamespace.Length - businessNamespaceLen - 1);
                        }
                    }
                    else
                    {
                        relativeNamespace = destinationNamespace;
                    }
                }
                else
                {
                    relativeNamespace = destinationNamespace;
                }
                relativePath = relativeNamespace.Replace('.', '/');
            }


            return relativePath;

        }

        [Obsolete("Does not appear to be in use. Consider removing.", false)]
        private static string NamespaceToFolderName(string classNamespace)
        {
            string returnPath = classNamespace.Replace('.', '\\');
            returnPath = String.Format(@"{0}\", returnPath);

            return returnPath;
        }


        /// <summary>
        /// Given a business entity, determines the namespace for the repository 
        /// and related classes that will be generated by the guidance package.
        /// </summary>
        /// <remarks>
        /// The following algorithm is applied:
        /// <para></para>
        /// If DataAccessProject is VB, Then Target namespace = entityName + "Repository"
        /// <list type="bullet">
        /// <item>If DataAccessProject is CSharp, 
        /// Then Target namespace = dataAccessNamespace + entityName + "Repository"</item>
        /// <item>If DataAccessProject is VB, 
        /// Then Target namespace = entityName + "Repository"</item>
        /// </list>
        /// </remarks>
        public static string ResolveNamespace(bool dataAccessProjectIsVB, string dataAccessNamespace, string entityName)
        {
            if (String.IsNullOrEmpty(dataAccessNamespace)) throw new ArgumentNullException("dataAccessNamespace");
            if (String.IsNullOrEmpty(entityName)) throw new ArgumentNullException("entityName");

            if (dataAccessProjectIsVB)
                return String.Format(CultureInfo.InvariantCulture,
                    "{0}DataAccess", entityName);

            return String.Format(CultureInfo.InvariantCulture,
                    "{0}.{1}DataAccess", dataAccessNamespace, entityName);
        } 


        [Obsolete("No longer in use. Consider removing.", false)]
        private static string GetCommonPrefix(string entitiesNamespace, string dataAccessNamespace)
        {
            string prefix = string.Empty;
            bool stop = false;
            int lastIndex = -1;
            if (string.Compare(entitiesNamespace, dataAccessNamespace, true) != 0)
            {
                while (!stop)
                {
                    lastIndex = entitiesNamespace.Substring(prefix.Length).IndexOf('.');
                    if (lastIndex != -1)
                    {
                        string temp = entitiesNamespace.Substring(0, lastIndex + prefix.Length + 1);
                        if (dataAccessNamespace.Contains(temp))
                        {
                            prefix = temp;
                        }
                        else
                        {
                            stop = true;
                        }
                    }
                    else
                    {
                        stop = true;
                    }
                }
            }
            else
            {
                prefix = dataAccessNamespace;
            }
            return prefix;
        }

        /// <summary>
        /// Returns the appropriate repository namespace given a database provider.
        /// </summary>
        /// <param name="databaseProvider"></param>
        /// <returns></returns>
        public static string ResolveRepositoryDatabaseNamespace(string databaseProvider)
        {
            if (String.IsNullOrEmpty(databaseProvider)) throw new ArgumentNullException("databaseProvider");

            if (String.Equals(databaseProvider, DatabaseProvider.SqlServer))
                return "Microsoft.Practices.Repository.SqlServer";

            if (String.Equals(databaseProvider, DatabaseProvider.MsOra) ||
                String.Equals(databaseProvider, DatabaseProvider.ODP))
                return "Microsoft.Practices.Repository.Oracle";

            throw new Exception(
                String.Format("Database provider {0} not supported. Supported providers include:\n{1}\n{2}\n{3}"
                , databaseProvider, DatabaseProvider.SqlServer, DatabaseProvider.MsOra, DatabaseProvider.ODP));
        }

        /// <summary>
        /// Determines the business entity namespace.
        /// </summary>
        /// <remarks>
        /// The business entity namespace is determined as follows.
        /// <para></para>
        /// If the entity.Namespace property is not null or empty, then it is returned as the namespace.
        /// If the entity.Namespace property is null or empty, then the default namespace (for CSharp projects)
        /// or root namespace (for VB projects) is returned.
        /// </remarks>
        public static string ResolveBusinessEntityNamespace(Entity entity, String businessEntityDefaultOrRootNamespace)
        {
            if (entity == null) throw new ArgumentNullException("entity");

            if (!String.IsNullOrEmpty(entity.Namespace)) return entity.Namespace;
            return businessEntityDefaultOrRootNamespace;            
        }
    }
}
